<?php
namespace Modules\Relatorio\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource;
use Teknisa\Model\Suprimentos\Entities\Prmarcpedcol;
use Teknisa\Model\Suprimentos\Entities\Itempedcol;
use Teknisa\Model\Suprimentos\Entities\Pedcolgeral;
use Teknisa\Model\Geral\Entities\Lista;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Sup03100EvolucaoPrecosService {

    protected $entityManager;
    protected $environment;
    protected $dataSourceManager;
    protected $newCode;

    public function __construct(EntityManager $entityManager, $environment, DataSource\Manager $dataSourceManager, $newCode) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCode = $newCode;
    }

    public function iniciaTransacao() {
        $this->entityManager->beginTransaction();
    }

    public function commit() {
        $this->entityManager->commit();
    }

    public function flush() {
        $this->entityManager->flush();
    }

    public function rollback() {
        $this->entityManager->rollback();
    }

    public function setTrabEvolPrec($conditions) {
        try {
	        $this->iniciaTransacao();

            $connection = $this->entityManager->getConnection();
            $sql = "INSERT INTO TRABEVOLPREC (NREVOLPRECO,CDPRODUTO,DTINIPEREVOL, DTFINPEREVOL,VRPRECOEVOL,IDREGEVOLINC) SELECT :P_NREVOLPRECO, PRODENTR.CDPRODESTO, PERIODO.DATAINI, PERIODO.DATAFIN, SUM(VRPRECUNI)/SUM(QTPRODEST) AS VRFINAL, 'N' AS IDREGEVOLINC FROM (  SELECT TO_DATE(:P_DTINI) DATAINI, (TO_DATE(:P_DTINI) + (:P_NRPERI * 1)) -1 DATAFIN FROM DUAL UNION SELECT TO_DATE(:P_DTINI) + (:P_NRPERI * 1) DATAINI, (TO_DATE(:P_DTINI) + (:P_NRPERI * 2)) - 1 DATAFIN FROM DUAL UNION SELECT TO_DATE(:P_DTINI) + (:P_NRPERI * 2) DATAINI, (TO_DATE(:P_DTINI) + (:P_NRPERI * 3)) - 1 DATAFIN FROM DUAL UNION SELECT TO_DATE(:P_DTINI) + (:P_NRPERI * 3) DATAINI, (TO_DATE(:P_DTINI) + (:P_NRPERI * 4)) - 1 DATAFIN FROM DUAL ) PERIODO, (  SELECT PRODUTO.CDPRODESTO, ITEMAUTF.DTPROGENTAF, SUM(ITEMAUTF.QTPROGENTAF * PRODUTO.VRFATOCONV) QTPRODEST, SUM(ITEMAUTF.QTPROGENTAF * ITEMAUTF.VRPREUNIAF) VRPRECUNI FROM ITEMAUTF INNER JOIN AUTOFORNE ON ITEMAUTF.CDFILDEST  = AUTOFORNE.CDFILDEST AND ITEMAUTF.CDFILLANC  = AUTOFORNE.CDFILLANC AND ITEMAUTF.NRAUTOFORN = AUTOFORNE.NRAUTOFORN INNER JOIN FILIAL    ON FILIAL.CDFILIAL     = ITEMAUTF.CDFILDEST INNER JOIN PRODUTO   ON ITEMAUTF.CDPRODUTO  = PRODUTO.CDPRODUTO WHERE FILIAL.CDREGIAO = :P_CDREGIAO AND PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND RPAD(:P_CDPRODFIN, 20, 'Z') AND ITEMAUTF.DTPROGENTAF BETWEEN :P_DTINI AND :P_DTFIN AND ((AUTOFORNE.IDAUTOFORNEX = 'N' AND :P_AFEXTRA = 'N') OR (AUTOFORNE.IDAUTOFORNEX = 'S' AND :P_AFEXTRA = 'S')) AND NVL(AUTOFORNE.IDBAIXAAF, '0') <> '5' AND NVL(AUTOFORNE.IDBAIXAAF, '0') <> '6' AND NVL(AUTOFORNE.IDBAIXAAF, '0') <> '4' GROUP BY PRODUTO.CDPRODESTO, ITEMAUTF.DTPROGENTAF ) PRODENTR WHERE PRODENTR.DTPROGENTAF BETWEEN PERIODO.DATAINI AND PERIODO.DATAFIN GROUP BY PRODENTR.CDPRODESTO, PERIODO.DATAINI, PERIODO.DATAFIN";
            $query = $this->entityManager->getConnection()->prepare($sql);

            $stProximoCodigo = $this->newCode->getCodeByTableName('EVOLPRECO', $this->environment->getNrOrg());

            $query->bindValue("P_DTINI",        $conditions['DTPERIOINI']);
            $query->bindValue("P_DTFIN",        $conditions['DTPERIOFIN']);
            $query->bindValue("P_NRPERI",       $conditions['NRPERIODO']);
            $query->bindValue("P_CDREGIAO",     $conditions['CDREGIAO']);
            $query->bindValue("P_CDPRODINI",    $conditions['CDPRODINI']);
            $query->bindValue("P_CDPRODFIN",    $conditions['CDPRODFIN']);
            $query->bindValue("P_AFEXTRA",      $conditions['IDAFEXTRA']);
            $query->bindValue("P_NREVOLPRECO",  $stProximoCodigo);

            $query->execute();
            $this->commit();

            return $stProximoCodigo;

        } catch (\Exception $e) {
		    $this->rollback();
		    throw $e;
		}
    }

    public function getConditionsFromFilter($filter) {
        try {
            $conditions = array();
            foreach($filter->getConditions() as $filter) {
                if(is_array($filter['value']))
                    $conditions[$filter['columnName']] = implode("_", $filter['value']) . '_';
                else
                    $conditions[$filter['columnName']] = $filter['value'];
            }
            return $conditions;
        } catch (\Exception $e) {
		    throw $e;
		}
    }

    public function setEvolucaoPrecos($nrevolpreco, $conditions) {
        try {
	        $this->iniciaTransacao();

            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'EVOLUCAO_PRECOS');
            $procedure->addParam(new Param('P_NREVOLPRECO', Param::PARAM_INPUT,  $nrevolpreco,              Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINI',       Param::PARAM_INPUT,  $conditions['DTPERIOINI'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFIN',       Param::PARAM_INPUT,  $conditions['DTPERIOFIN'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPERI',      Param::PARAM_INPUT,  $conditions['NRPERIODO'],  Param::PARAM_TYPE_STR));

            $procedure->execute();

            $this->commit();

        } catch (\Exception $e) {
		    $this->rollback();
		    throw $e;
		}
    }

    public function getEvolucaoPrecosProduto($nrevolpreco) {
        try {
            $sql = "SELECT :P_NREVOLPRECO AS NREVOLPRECO, TRABEVOLPREC.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, TRABEVOLPREC.NREVOLPRECO, TRABEVOLPREC.IDREGEVOLINC FROM PRODUTO INNER JOIN TRABEVOLPREC ON TRABEVOLPREC.CDPRODUTO = PRODUTO.CDPRODUTO WHERE TRABEVOLPREC.NREVOLPRECO = :P_NREVOLPRECO GROUP BY TRABEVOLPREC.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, TRABEVOLPREC.NREVOLPRECO, TRABEVOLPREC.IDREGEVOLINC ORDER BY PRODUTO.CDARVPROD";

            $params = array(
                "P_NREVOLPRECO" => $nrevolpreco
            );

        	return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
		    throw $e;
		}
    }

}